#define M_PI 3.14159265359f

void CreateViewMatrix(D3DMATRIX& mat,D3DVECTOR& vf,D3DVECTOR& va, float roll=0.0f);
void CreateProjectionMatrix(D3DMATRIX& mat, float near_plane, float far_plane, float fov);
void CreateUnitMatrix(D3DMATRIX& mat);
void CreateZeroMatrix(D3DMATRIX& mat);
void CreateTranslateMatrix(D3DMATRIX& mat,D3DVALUE x,D3DVALUE y,D3DVALUE z);
void CreateScaleMatrix(D3DMATRIX& mat,D3DVALUE x,D3DVALUE y,D3DVALUE z);
void CreateRotateXMatrix(D3DMATRIX& mat,D3DVALUE deg);
void CreateRotateYMatrix(D3DMATRIX& mat,D3DVALUE deg);
void CreateRotateZMatrix(D3DMATRIX& mat,D3DVALUE deg);
void TransformVector(D3DMATRIX& mat,D3DVECTOR& v,D3DVECTOR& vt);
void MulMatrix(D3DMATRIX& m1,D3DMATRIX& m2,D3DMATRIX& mt);


/*-----------------------------------------------------
	_Wϊ}gbNX̍쐬
------------------------------------------------------*/
void CreateViewMatrix(D3DMATRIX& mat,D3DVECTOR& vf,D3DVECTOR& va, float roll)
{
	D3DVECTOR world_up(0,1,0);

	CreateUnitMatrix(mat);

	D3DVECTOR up, right, view_dir;
	view_dir = Normalize(va - vf);
	right = CrossProduct(world_up, view_dir);
	up = CrossProduct(view_dir, right);

	right = Normalize(right);
	up = Normalize(up);
	
	mat(0, 0) = right.x;
	mat(1, 0) = right.y;
	mat(2, 0) = right.z;
	mat(0, 1) = up.x;
	mat(1, 1) = up.y;
	mat(2, 1) = up.z;
	mat(0, 2) = view_dir.x;
	mat(1, 2) = view_dir.y;
	mat(2, 2) = view_dir.z;

	mat(3, 0) = -DotProduct(right, vf);
	mat(3, 1) = -DotProduct(up, vf);
	mat(3, 2) = -DotProduct(view_dir, vf);

	if (roll != 0.0f) {
		D3DMATRIX zmat;
		CreateRotateZMatrix(zmat, -roll);
		MulMatrix(zmat, mat, mat);
	}
}

/*----------------------------------------------------
	ϊ}gbNX̍쐬
-------------------------------------------------------*/
void CreateProjectionMatrix(D3DMATRIX& mat, float near_plane, float far_plane, float fov)
{
	float	c, s, Q;
	c = (float) cos(fov*0.5);
	s = (float) sin(fov*0.5);
	Q = s/(1.0f - near_plane/far_plane);

	CreateZeroMatrix(mat);
	mat(0, 0) = c;
	mat(1, 1) = c;
	mat(2, 2) = Q;
	mat(3, 2) = -Q*near_plane;
	mat(2, 3) = s;
}

/*-------------------------------------------------------
	Pʃ}gbNX
----------------------------------------------------------*/
void CreateUnitMatrix(D3DMATRIX& mat)
{
	int i,j;

	for(i=0;i<4;i++){
		for(j=0;j<4;j++){
			mat(i,j) = 0;
		}
		mat(i,i) = 1;
	}
}

/*--------------------------------------------------------
	}gbNXNA
-----------------------------------------------------------*/
void CreateZeroMatrix(D3DMATRIX& mat)
{
	int i,j;

	for(i=0;i<4;i++){
		for(j=0;j<4;j++){
			mat(i,j) = 0;
		}
	}
}

/*---------------------------------------------------------
	sړ
----------------------------------------------------------*/
void CreateTranslateMatrix(D3DMATRIX& mat,D3DVALUE x,D3DVALUE y,D3DVALUE z)
{
	CreateUnitMatrix(mat);

	mat(3,0) = x;
	mat(3,1) = y;
	mat(3,2) = z;
}

/*---------------------------------------------------------
	XP[
-----------------------------------------------------------*/
void CreateScaleMatrix(D3DMATRIX& mat,D3DVALUE x,D3DVALUE y,D3DVALUE z)
{
	CreateUnitMatrix(mat);

	mat(0,0) = x;
	mat(1,1) = y;
	mat(2,2) = z;
}

/*---------------------------------------------------------
	w̉]
----------------------------------------------------------*/
void CreateRotateXMatrix(D3DMATRIX& mat,D3DVALUE deg)
{
	D3DVALUE c,s;

	CreateUnitMatrix(mat);

	c = D3DVAL(cos(deg));
	s = D3DVAL(sin(deg));

	mat(1,1) = c;
	mat(1,2) = s;
	mat(2,1) = -s;
	mat(2,2) = c;
}

/*---------------------------------------------------------
	x̉]
----------------------------------------------------------*/
void CreateRotateYMatrix(D3DMATRIX& mat,D3DVALUE deg)
{
	D3DVALUE c,s;

	CreateUnitMatrix(mat);

	c = D3DVAL(cos(deg));
	s = D3DVAL(sin(deg));

	mat(0,0) = c;
	mat(0,2) = -s;
	mat(2,0) = s;
	mat(2,2) = c;
}

/*---------------------------------------------------------
	ỷ]
----------------------------------------------------------*/
void CreateRotateZMatrix(D3DMATRIX& mat,D3DVALUE deg)
{
	D3DVALUE c,s;

	CreateUnitMatrix(mat);

	c = D3DVAL(cos(deg));
	s = D3DVAL(sin(deg));

	mat(0,0) = c;
	mat(0,1) = s;
	mat(1,0) = -s;
	mat(1,1) = c;
}

/*----------------------------------------------------------
	ϊ
----------------------------------------------------------*/
void TransformVector(D3DMATRIX& mat,D3DVECTOR& v,D3DVECTOR& vt)
{
	D3DVALUE w;

	vt.x = mat(0,0) * v.x + mat(1,0) * v.y + mat(2,0) * v.z + mat(3,0);
	vt.y = mat(0,1) * v.x + mat(1,1) * v.y + mat(2,1) * v.z + mat(3,1);
	vt.z = mat(0,2) * v.x + mat(1,2) * v.y + mat(2,2) * v.z + mat(3,2);
	w = mat(0,3) * v.x + mat(1,3) * v.y + mat(2,3) * v.z + mat(3,3);

	vt.x /= w;
	vt.y /= w;
	vt.z /= w;
}

/*-----------------------------------------------------------
	}gbNX̏Z
-------------------------------------------------------------*/
void MulMatrix(D3DMATRIX& m1,D3DMATRIX& m2,D3DMATRIX& mt)
{
	int i,j;

	for(i=0;i<4;i++){
		for(j=0;j<4;j++){
			mt(i,j) = m1(i,0) * m2(0,j) + m1(i,1) * m2(1,j) + m1(i,2) * m2(2,j) + m1(i,3) * m2(3,j);
		}
	}
}

//_f[^͊֐
void LVertexSet(D3DLVERTEX &v, float x, float y, float z, long color, float tu, float tv) 
{
	v.x = x;
	v.y = y;
	v.z = z;
	v.color = color;
	v.specular = D3DRGB(0,0,0);
	v.tu = tu;
	v.tv = tv;
}

void VertexSet(D3DVERTEX &v, float x, float y, float z, float nx, float ny, float nz, float tu, float tv) 
{
	v.x  = x;
	v.y  = y;
	v.z  = z;
	v.nx = nx;
	v.ny = ny;
	v.nz = nz;
	v.tu = tu;
	v.tv = tv;
}

